<?php
//
//
//  NAME
//
//    class htsearch
//
//
//  DESCRIPTION
//
//    Wrapper class for the ht://Dig search engine.
//
//
//  INSTALLATION
//
//    Configure ht://Dig to use the template files that come with
//    this wrapper instead of the standard templates.
//
//    Apply the "htsearch.patch" to the ht::/Dig source tree and
//    recompile.  This fixes a problem with additional newline
//    characters being emitted by the htsearch program which mess
//    up the result array.  Should be fixed in ht://Dig 3.2.x.
//
//
//  USEAGE
//
//    - Instantiate a htsearch object with "new".
//    - Execute a htsearch query using the query() method.
//    - Handle the search results using the methods
//      + match()          -> return match n on current page
//      + matches_total()  -> number of matching documents in total
//      + matches_first()  -> index of first match on current page
//      + matches_last()   -> index of last match on current page
//      + pages_total()    -> number of pages in total
//      + page_nr()        -> number of current page
//
//    To make the class work with other display templates than the
//    provided "short.html" and "long.html", a new class needs to
//    be derived which implements another kind of match() method.
//
//    Additional formats can be introduced in derived classed.
//
//
//  EXAMPLE
//
//    $foo = new htsearch("/opt/www/cgi-bin/htsearch","myconfig");
//    $foo->query("bar");
//    for( $i = $foo->matches_first();
//         $i <= $foo->matches_last();
//         $i++ ) {
//      echo($foo->match($i)."\n");
//    }
//
//
//  AUTHORS
//
//    Torsten Neuer  <tneuer@inwise.de>
//
//
//  BUGS
//
//    - Does not yet handle dynamic htsearch parameters configured
//      with the "allow_in_form" configuration tag.
//
//

class htsearch {
  var $htsearch_bin;          // string
  var $config_data;           // string
  var $format_data;           // string
  var $method_data;           // string
  var $matches_data;          // int
  var $exclude_data;          // string
  var $restrict_data;         // string
  var $sort_data;             // string
  var $result_data;           // array
  var $result_count;          // int
  var $matches_per_page_data; // int
  var $matches_total_data;    // int
  var $matches_start_data;    // int
  var $matches_end_data;      // int
  var $pages_total_data;      // int
  var $pages_current_data;    // int

  //
  //  Set the "config" parameter value for htsearch.
  //  The value is url-encoded and shell-escaped for obvious reasons.
  //  Returns 0.
  //
  function set_config( $new_config ) {                           // = int
    $this->config_data = EscapeShellCmd(UrlEncode($new_config));
    return 0;
  } // set_config()

  //
  //  Set the "sort" parameter value for htsearch.
  //  Accepts only valid values.
  //  Returns 0 on success, 1 on failure.
  //
  function set_sort( $new_sort ) {                               // = int
    $the_sort = strtolower($new_sort);
    if( !strcmp($the_sort,"score")    ||
	!strcmp($the_sort,"date")     ||
	!strcmp($the_sort,"time")     ||
	!strcmp($the_sort,"title")    ||
        !strcmp($the_sort,"revscore") ||
	!strcmp($the_sort,"revdate")  ||
	!strcmp($the_sort,"revtime")  ||
	!strcmp($the_sort,"revtitle") ) {
      $this->sort_data = $the_sort;
      return 0;
    }
    return 1;
  } // set_sort()

  //
  //  Set the "format" parameter value for htsearch.
  //  Builtin formats are converted to non-builtin.
  //  Returns 0 on success.
  //
  function set_format( $new_format ) {                           // = int
    $the_format = strtolower($new_format);
    if( !strcmp($the_format,"builtin-long") ) {
      $this->format_data = "long";
      return 0;
    } else if( !strcmp($the_format,"builtin-short") ) {
      $this->format_data = "short";
      return 0;
    }
    $this->format_data = $the_format;
    return 0;
  } // set_format()

  //
  //  Set the "method" parameter value for htsearch.
  //  Only valid values are accepted.
  //  Returns 0 on success, 1 on failure.
  //
  function set_method( $new_method ) {                           // = int
    $the_method = strtolower($new_method);
    if( !strcmp($the_method,"or")  ||
	!strcmp($the_method,"and") ||
	!strcmp($the_method,"boolean") ) {
      $this->method_data = $the_method;
      return 0;
    } else if( !strcmp($the_method,"all") ) {
      $this->method_data = "and";
      return 0;
    } else if( !strcmp($the_method,"any") ) {
      $this->method_data = "or";
      return 0;
    } else if( !strcmp($the_method,"bool") ) {
      $this->method_data = "boolean";
      return 0;
    }
    return 1;
  } // set_method()

  //
  //  Set the "exclude" parameter value for htsearch.
  //  The value is url-encoded and shell-escaped for obvious reasons.
  //  Returns 0.
  //
  function set_exclude( $new_exclude ) {                         // = int
    $this->exclude_data = EscapeShellCmd(UrlEncode($new_exclude));
    return 0;
  } // set_exclude()

  //
  //  Set the "restrict" parameter value for htsearch.
  //  The value is url-encoded and shell-escaped for obvious reasons.
  //  Returns 0.
  //
  function set_restrict( $new_restrict ) {                       // = int
    $this->restrict_data = EscapeShellCmd(UrlEncode($new_restrict));
    return 0;
  } // set_restrict()

  //
  //  Set the "matches_per_page" parameter value for htsearch.
  //  The value passed to this function may be of any type but must
  //  be convertable to an integer greater than 0.
  //  Returns 0 on success, 1 on failure.
  //
  function set_matches_per_page( $new_matches_per_page ) {       // = int
    $the_matches_per_page = ((int)$new_matches_per_page);
    if( $the_matches_per_page > 0 ) {
      $this->matches_per_page_data = $the_matches_per_page;
      return 0;
    }
    return 1;
  } // set_matches_per_page()

  //
  //  Initialize the search wrapper class with default htsearch
  //  path and parameter values.  Most values are set via the
  //  safe set_XXX() functions of this class.
  //  
  function htsearch( $def_htsearch = "/opt/www/cgi-bin/htsearch",
		     $def_config = "htdig",
		     $def_sort = "score",
		     $def_format = "long",
		     $def_method = "and",
		     $def_matches = 20,
		     $def_exclude = "",
		     $def_restrict = "" ) {                      // constructor
    $this->htsearch_bin = $def_htsearch;
    $this->set_config($def_config);
    $this->set_sort($def_sort);
    $this->set_format($def_format);
    $this->set_method($def_method);
    $this->set_matches_per_page($def_matches);
    $this->set_exclude($def_exclude);
    $this->set_restrict($def_restrict);
    $this->result_count = -1;
  } // htsearch()

  //
  //  Return the value of the "config" parameter for htsearch.
  //
  function config() {                                            // = string
    return $this->config_data;
  } // config()

  //
  //  Return the value of the "sort" parameter for htsearch.
  //
  function sort() {                                              // = string
    return $this->sort_data;
  } // sort()

  //
  //  Return the value of the "format" parameter for htsearch.
  //
  function format() {                                            // = string
    return $this->format_data;
  } // format()

  //
  //  Return the value of the "method" parameter for htsearch.
  //
  function method() {                                            // = string
    return $this->method_data;
  } // method()

  //
  //  Return the value of the "exclude" parameter for htsearch.
  //
  function exclude() {                                           // = string
    return $this->exclude_data;
  } // exclude()

  //
  //  Return the value of the "restrict" parameter for htsearch.
  //
  function restrict() {                                          // = string
    return $this->restrict_data;
  } // restrict()

  //
  //  Return the value of the "matches_per_page" parameter for htsearch.
  //
  function matches_per_page() {                                  // = int
    return $this->matches_per_page_data;
  } // matches_per_page()

  //
  //  Execute htsearch for the given keywords, words and page.
  //  Returns 0 if no match is found, -1 on syntax errors, -2
  //  for other errors or >0 indicating the number of documents
  //  found by this query.
  //
  function query( $the_keywords,
		  $the_words,
		  $the_page = 1 ) {                              // = int
    $page_val = (int)$the_page;
    if( ($page_val = ((int)$the_page)) < 1 ) {
      return -2; // invalid page number
    }
    exec($this->htsearch_bin." \"".
	 "config=".$this->config()."&".
	 "words=".EscapeShellCmd(UrlEncode($the_words))."&".
	 "keywords=".EscapeShellCmd(UrlEncode($the_keywords))."&".
	 "page=".((string)$page_val)."&".
	 "sort=".$this->sort()."&".
	 "format=".$this->format()."&".
	 "method=".$this->method()."&".
	 "matchesperpage=".$this->matches_per_page()."&".
	 "exclude=".$this->exclude()."&".
	 "restrict=".$this->restrict()."\"",
	 &$this->result_data);
    $this->result_count = count($this->result_data);
    if( $this->result_count < 2 ) {
      return -2;                   // error executing htsearch
    } else if( !strcmp($this->result_data[2],"SYNTAX") ) {
      return -1;                   // syntax error
    } else if( !strcmp($this->result_data[2],"NOMATCH") ) {
      return 0;                    // no matching document
    }
    $this->matches_total_data = ((int)$this->result_data[2]);
    $this->matches_start_data = ((int)$this->result_data[3]);
    $this->matches_end_data = ((int)$this->result_data[4]);
    $this->pages_current_data = ((int)$this->result_data[5]);
    $this->pages_total_data = ((int)$this->result_data[6]);
    return ($this->result_count - 7);
  } // query()

  //
  //  Returns a string value for the result of a query().
  //
  function result_name( $search_result ) {                       // = string
    if( $search_result < -1 ) {
      return "ERROR";
    } else if( $search_result == -1 ) {
      return "SYNTAX";
    } else if( $search_result != 0 ) {
      return "OK";
    }
    return "NOMATCH";
  } // result_name()

  //
  //  Return the total number of matching documents.
  //
  function matches_total() {                                     // = int
    return $this->matches_total_data;
  } // matches_total()

  //
  //  Return the number of the first match in the current query.
  //
  function matches_first() {                                     // = int
    return $this->matches_start_data;
  } // matches_first()

  //
  //  Return the number of the last match in the current query.
  //
  function matches_last() {                                      // = int
    return $this->matches_end_data;
  } // matches_last()

  //
  //  Return a match from the list of matching documents or
  //  an empty string if the index is out of range (which is
  //  matches_first()..matches_last()).
  //
  function match( $the_match ) {                                 // = string
    $m = (int)$the_match;
    if( $m &&
	($m >= $this->matches_first()) &&
	($m <= $this->matches_last()) ) {
      return $this->result_data[7+($m-$this->matches_first())];
    }
    return "";
  } // match()

  //
  //  Return the page number for the current query.
  //
  function page_nr() {                                           // = int
    return $this->pages_current_data;
  } // page_nr()

  //
  //  Return the number of pages in total for teh current query.
  //
  function pages_total() {                                      // = int
    return $this->pages_total_data;
  } // pages_total()

} // htsearch

?>
